@@ -4,6 +4,9 @@ Python library to communicate to the AtMega32 uC running 'eyes.c'
 Author  : Ajith Kumar B.P, bpajith@gmail.com
 License : GNU GPL version 3
 Started on 1-Nov-2010
+Last Edit : 13-Oct-2011   : Added MCP2200 support (for version 2)
+Last Edit : 4-Nov-2011    : DAC maximum set to 5.000 volts
+
 
 The hardware consisists of :
 1) 2 Digital Inputs
@@ -23,7 +26,7 @@ The hardware consisists of :
 12)2 Inverting amplifiers with gain = 47 , mainly used for microphones. 
 '''
 
-import serial, struct, math, time, commands
+import serial, struct, math, time, commands, sys, os
 
 #Commands with One byte argument (41 to 80) 
 GETVERSION  =   1
@@ -51,6 +54,7 @@ ADC2CMP		=  57	# Route ADC input to ACOMP-
 SETPWM		=  58	# Set 488 Hz PWM wave on TC0
 SETPWMDAC	=  59	# Set 31.25 kHz PWM wave on TC0
 GETPORT		=  60	# PINX data from port X
+IRSEND      =  61   # Send 8 bit data on SQR1, using infrared LED
 
 # Commands with Two bytes argument (81 to 120)
 SETPWM0		=  81	# PWM on on OSC0
@@ -91,9 +95,9 @@ AWAITFALL	= 8
 BUFSIZE     = 1800       # status + adcinfo + 1800 data
 
 #Serial devices to search for EYES hardware.  
-device_list = ['/dev/ttyUSB0', '/dev/ttyUSB1', '/dev/ttyUSB2', '/dev/ttyUSB3',\
-		'/dev/tts/USB0','/dev/tts/USB1',\
-		'COM2','COM3','COM4','COM5','COM6','COM7','COM8','COM9','COM10','COM11','COM12',1,2,3]
+linux_list = ['/dev/ttyUSB0', '/dev/ttyUSB1', '/dev/ttyUSB2', '/dev/ttyUSB3', '/dev/tts/USB0','/dev/tts/USB1', \
+		'/dev/ttyACM0','/dev/ttyACM1']
+
 
 def open(dev = None):
 	'''
@@ -106,11 +110,10 @@ def open(dev = None):
 	print 'Could not find Phoenix-EYES hardware'
 	print 'Check the connections.'
 
-DACMAX = 4.993				# MCP4922 DAC goes only up to 4.933 volts
+DACMAX = 5.000				# MCP4922 DAC goes only up to 4.933 volts, in version 1
 BAUDRATE = 38400			# Serial communication
 
 class Eyes:
-	#buf = array.array('B',BUFSIZE * [0])    # unsigned character array, Global
 	fd = None								# init should fill this
 	adcsize = 1
 	m = [10.0/4095]*2 + [5.0/4095]*6 + [4095./DACMAX/2, 4095.0/DACMAX] # 8th and 9th are for DAC
@@ -125,7 +128,18 @@ class Eyes:
 		whether a port is already open or not, without doing any transactions to it.
 		"""
 		self.adcsize = 2;
-	
+		
+		if os.name == 'nt':	
+			device_list = []
+			for k in range(1,255):
+				s = 'COM%d'%k
+				device_list.append(s)
+			for k in range(1,11):
+				device_list.append(k)
+		else:
+			device_list = linux_list
+		
+
 		for dev in device_list:
 			try:
 				handle = serial.Serial(dev, BAUDRATE, stopbits=1, timeout = 0.3, parity=serial.PARITY_EVEN)
@@ -137,7 +151,10 @@ class Eyes:
 				continue
 			print 'and opened. ',
 			handle.flush()
-			handle.flushInput()
+			time.sleep(.5)
+			while handle.inWaiting() > 0 :
+				print 'inWaiting'
+				handle.flushInput()
 			handle.write(chr(GETVERSION))
 			res = handle.read(1)
 			ver = handle.read(5)		# 5 character version number
@@ -151,6 +168,11 @@ class Eyes:
 			else:
 				print 'No EYES hardware detected'
 				self.fd = None
+#------------------------------------------------------------------------------------
+
+	def dwrite(self,ch):
+		self.fd.write(ch)
+		time.sleep(0.01)		#MCP2200 to ATmega transfer has no handshake
 
 #-------------------- Pulse Width Modulated Waveform on TC0 and TC2 ------------------
 	def set_pwmdac(self, vout):        # Value in 0 to 5V
@@ -161,8 +183,8 @@ class Eyes:
 		'''
 		if 0 <= vout <= 5.0:
 			val = int(vout*255.0/5.0)
-			self.fd.write(chr(SETPWMDAC))
-			self.fd.write(chr(val))
+			self.dwrite(chr(SETPWMDAC))
+			self.dwrite(chr(val))
 			self.fd.read(1)
 			return val * 5.0 / 255
 
@@ -173,12 +195,17 @@ class Eyes:
 		'''
 		if 0 <= ds <= 100:
 			val = int(ds*255.0/100)
-			self.fd.write(chr(SETPWM))
-			self.fd.write(chr(val))
+			self.dwrite(chr(SETPWM))
+			self.dwrite(chr(val))
 			self.fd.read(1)
 			return val * 100.0 / 255
 
 #---------------- Square Wave Generation & Measuring the Frequency ------------------
+	def irsend(self, dat):				# Infrared transmission
+			self.dwrite(chr(IRSEND))
+			self.dwrite(chr(dat))
+			self.fd.read(1)
+
 	def set_sqr0(self, freq):        # Sets Squarewave on the PULSE output
 		'''
 		Sets a square wave on the PULSE output. Frequency from 15Hz to 40000000 Hz (4 MHz), but 
@@ -186,9 +213,9 @@ class Eyes:
 		The function sets the nearest possible value and returns it.
 		'''
 		if freq < 1:        # Disable squarewave on PULSE
-			self.fd.write(chr(SETCOUNTER0))
-			self.fd.write(chr(0))
-			self.fd.write(chr(0))
+			self.dwrite(chr(SETCOUNTER0))
+			self.dwrite(chr(0))
+			self.dwrite(chr(0))
 			self.fd.read(1)
 			return 0
 
@@ -203,9 +230,9 @@ class Eyes:
 			setpoint = 255
 		OCR0 = int(setpoint)-1
 		#print clock_sel, OCR2
-		self.fd.write(chr(SETCOUNTER0))
-		self.fd.write(chr(clock_sel))
-		self.fd.write(chr(OCR0))
+		self.dwrite(chr(SETCOUNTER0))
+		self.dwrite(chr(clock_sel))
+		self.dwrite(chr(OCR0))
 		res = self.fd.read(1)
 		if res != 'D':
 			return None
@@ -221,9 +248,9 @@ class Eyes:
 		The function sets the nearest possible value and returns it.
 		'''
 		if freq < 1:        # Disable PWG
-			self.fd.write(chr(SETCOUNTER2))
-			self.fd.write(chr(0))
-			self.fd.write(chr(0))
+			self.dwrite(chr(SETCOUNTER2))
+			self.dwrite(chr(0))
+			self.dwrite(chr(0))
 			self.fd.read(1)
 			return 0
 
@@ -238,9 +265,9 @@ class Eyes:
 			setpoint = 255
 		OCR2 = int(setpoint)-1
 		#print clock_sel, OCR2
-		self.fd.write(chr(SETCOUNTER2))
-		self.fd.write(chr(clock_sel))
-		self.fd.write(chr(OCR2))
+		self.dwrite(chr(SETCOUNTER2))
+		self.dwrite(chr(clock_sel))
+		self.dwrite(chr(OCR2))
 		res = self.fd.read(1)
 		if res != 'D':
 			return None
@@ -345,10 +372,10 @@ class Eyes:
 		Used for storing the calibration constants of ADC and DAC.
 		WARNING: Using this function may destroy the Calibration Data.
 		'''
-		self.fd.write(chr(WREEPROM))
-		self.fd.write(chr(addr&255))
-		self.fd.write(chr(addr>>8))
-		self.fd.write(dat)
+		self.dwrite(chr(WREEPROM))
+		self.dwrite(chr(addr&255))
+		self.dwrite(chr(addr>>8))
+		self.dwrite(dat)
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'eeprom write byte error = ', res
@@ -358,10 +385,10 @@ class Eyes:
 		Reads 'nb' bytes starting from the specified address of the EEPROM memory of ATmega32.
 		Used for restoring the calibration constants of ADC and DAC.
 		'''
-		self.fd.write(chr(RDEEPROM))
-		self.fd.write(chr(addr&255))
-		self.fd.write(chr(addr>>8))
-		self.fd.write(chr(nb))
+		self.dwrite(chr(RDEEPROM))
+		self.dwrite(chr(addr&255))
+		self.dwrite(chr(addr>>8))
+		self.dwrite(chr(nb))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'eeprom read block error = ', res
@@ -401,6 +428,11 @@ class Eyes:
 		print
 		print 'LC: ch = %d m=%10.6f  c=%10.6f'%(ch, self.m[ch], self.c[ch])
 
+	def loadall_calib(self):
+		self.load_calib(0)
+		self.load_calib(1)
+		self.load_calib(8)
+
 #------------------------------------ ADC & DAC transactions -----------------------------
 
 	def set_current(self, i):
@@ -425,10 +457,10 @@ class Eyes:
 		'''
 		if (data > 4095): 		# DAC linearity problem
 			data = 4095
-		self.fd.write(chr(SETDAC))
-		self.fd.write(chr(ch))
-		self.fd.write(chr(data&255))
-		self.fd.write(chr(data>>8))
+		self.dwrite(chr(SETDAC))
+		self.dwrite(chr(ch))
+		self.dwrite(chr(data&255))
+		self.dwrite(chr(data>>8))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'WRITEDAC error ', res
@@ -465,8 +497,8 @@ class Eyes:
 		if (ch > 7):
 			print 'Argument error'
 			return
-		self.fd.write(chr(READADC))
-		self.fd.write(chr(ch))
+		self.dwrite(chr(READADC))
+		self.dwrite(chr(ch))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'READADC error ', res
@@ -483,8 +515,8 @@ class Eyes:
 		if (ch > 7):
 			print 'Argument error'
 			return
-		self.fd.write(chr(READADC))
-		self.fd.write(chr(ch))
+		self.dwrite(chr(READADC))
+		self.dwrite(chr(ch))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'WRITEDAC error ', res
@@ -502,8 +534,8 @@ class Eyes:
 		if (ch > 7):
 			print 'Argument error'
 			return
-		self.fd.write(chr(READADC))
-		self.fd.write(chr(ch))
+		self.dwrite(chr(READADC))
+		self.dwrite(chr(ch))
 		tm = time.time()				# Job is sent. Now mark the time
 		res = self.fd.read(1)
 		if res != 'D':
@@ -522,8 +554,8 @@ class Eyes:
 		if sam > 250:
 			print 'Sampling time MUST NOT exceed 250 microseconds'
 			return
-		self.fd.write(chr(SETSAMTIME))
-		self.fd.write(chr(sam))
+		self.dwrite(chr(SETSAMTIME))
+		self.dwrite(chr(sam))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'SETSAMTIME ERROR ', res
@@ -536,8 +568,8 @@ class Eyes:
 		if size > 2:
 			print 'ADC datasize MUST be 1 or 2 bytes'
 			return
-		self.fd.write(chr(SETADCSIZE))
-		self.fd.write(chr(size))
+		self.dwrite(chr(SETADCSIZE))
+		self.dwrite(chr(size))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'SETADCSIZE ERROR ', res
@@ -553,11 +585,11 @@ class Eyes:
 		if delay < 10:
 			return
 		if delay < 20:
-			self.fd.write(chr(QCAPTURE))
-			self.fd.write(chr(ch))
-			self.fd.write(chr(np&255))
-			self.fd.write(chr(np>>8))
-			self.fd.write(chr(delay))
+			self.dwrite(chr(QCAPTURE))
+			self.dwrite(chr(ch))
+			self.dwrite(chr(np&255))
+			self.dwrite(chr(np>>8))
+			self.dwrite(chr(delay))
 			st = time.time()
 			res = self.fd.read(1)
 			if res != 'D':
@@ -565,12 +597,12 @@ class Eyes:
 				return 0,0
 			asize = 1					# adc datasize = 1 for QCAPTURE
 		else:
-			self.fd.write(chr(CAPTURE))
-			self.fd.write(chr(ch))
-			self.fd.write(chr(np&255))
-			self.fd.write(chr(np>>8))
-			self.fd.write(chr(delay&255))
-			self.fd.write(chr(delay>>8))
+			self.dwrite(chr(CAPTURE))
+			self.dwrite(chr(ch))
+			self.dwrite(chr(np&255))
+			self.dwrite(chr(np>>8))
+			self.dwrite(chr(delay&255))
+			self.dwrite(chr(delay>>8))
 			res = self.fd.read(1)
 			if res != 'D':
 				print 'CAPTURE error ', res
@@ -620,10 +652,10 @@ class Eyes:
 		if delay < 10:
 			return
 		if delay < 20:				# Fast Capture, datasize = 1 byte
-			self.fd.write(chr(QCAPTURE01))
-			self.fd.write(chr(np&255))
-			self.fd.write(chr(np>>8))
-			self.fd.write(chr(delay))
+			self.dwrite(chr(QCAPTURE01))
+			self.dwrite(chr(np&255))
+			self.dwrite(chr(np>>8))
+			self.dwrite(chr(delay))
 			res = self.fd.read(1)
 			if res != 'D':
 				print 'CAPTURE01 error ', res
@@ -631,11 +663,11 @@ class Eyes:
 			asize = 1
 			tg01 =  0.009			# 0.009 milliseconds between CH0 and CH1
 		else:						# A slow capture
-			self.fd.write(chr(CAPTURE01))
-			self.fd.write(chr(np&255))
-			self.fd.write(chr(np>>8))
-			self.fd.write(chr(delay&255))
-			self.fd.write(chr(delay>>8))
+			self.dwrite(chr(CAPTURE01))
+			self.dwrite(chr(np&255))
+			self.dwrite(chr(np>>8))
+			self.dwrite(chr(delay&255))
+			self.dwrite(chr(delay>>8))
 			res = self.fd.read(1)
 			if res != 'D':
 				print 'CAPTURE01 error ', res
@@ -680,12 +712,12 @@ class Eyes:
 		'''
 		if delay < 10:
 			return
-		self.fd.write(chr(CAPTURE_M32))
-		self.fd.write(chr(ch))
-		self.fd.write(chr(np&255))
-		self.fd.write(chr(np>>8))
-		self.fd.write(chr(delay&255))
-		self.fd.write(chr(delay>>8))
+		self.dwrite(chr(CAPTURE_M32))
+		self.dwrite(chr(ch))
+		self.dwrite(chr(np&255))
+		self.dwrite(chr(np>>8))
+		self.dwrite(chr(delay&255))
+		self.dwrite(chr(delay>>8))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'CAPTURE_M32 error ', res
@@ -712,9 +744,9 @@ class Eyes:
 		Disable all modifiers to the capture call. The capture will try to
 		do a self triggering on the ADC input.
 		'''
-		self.fd.write(chr(SETACTION))
-		self.fd.write(chr(0))
-		self.fd.write(chr(0))
+		self.dwrite(chr(SETACTION))
+		self.dwrite(chr(0))
+		self.dwrite(chr(0))
 		self.fd.read(1)
 
 	def enable_wait_high(self, pin):
@@ -725,9 +757,9 @@ class Eyes:
 			mask = 0
 		else:
 			mask = 1 << pin          
-		self.fd.write(chr(SETACTION))
-		self.fd.write(chr(AWAITHI))
-		self.fd.write(chr(mask))
+		self.dwrite(chr(SETACTION))
+		self.dwrite(chr(AWAITHI))
+		self.dwrite(chr(mask))
 		self.fd.read(1)
 
 	def enable_wait_rising(self, pin):
@@ -739,9 +771,9 @@ class Eyes:
 		else:
 			mask = 1 << pin          
 		print 'wait_rising ', AWAITRISE
-		self.fd.write(chr(SETACTION))
-		self.fd.write(chr(AWAITRISE))
-		self.fd.write(chr(mask))
+		self.dwrite(chr(SETACTION))
+		self.dwrite(chr(AWAITRISE))
+		self.dwrite(chr(mask))
 		self.fd.read(1)
 
 	def enable_wait_low(self, pin):
@@ -752,9 +784,9 @@ class Eyes:
 			mask = 0
 		else:
 			mask = 1 << pin          
-		self.fd.write(chr(SETACTION))
-		self.fd.write(chr(AWAITLO))
-		self.fd.write(chr(mask))
+		self.dwrite(chr(SETACTION))
+		self.dwrite(chr(AWAITLO))
+		self.dwrite(chr(mask))
 		self.fd.read(1)
 
 	def enable_wait_falling(self, pin):
@@ -766,9 +798,9 @@ class Eyes:
 		else:
 			mask = 1 << pin          
 		print 'wait_rising ', AWAITRISE
-		self.fd.write(chr(SETACTION))
-		self.fd.write(chr(AWAITFALL))
-		self.fd.write(chr(mask))
+		self.dwrite(chr(SETACTION))
+		self.dwrite(chr(AWAITFALL))
+		self.dwrite(chr(mask))
 		self.fd.read(1)
 
 	def enable_set_high(self, pin):
@@ -776,9 +808,9 @@ class Eyes:
 		Sets the speciied 'pin' HIGH, just before every Capture.
 		'''
 		mask = 1 << pin
-		self.fd.write(chr(SETACTION))
-		self.fd.write(chr(ASET))
-		self.fd.write(chr(mask))
+		self.dwrite(chr(SETACTION))
+		self.dwrite(chr(ASET))
+		self.dwrite(chr(mask))
 		self.fd.read(1)
 
 	def enable_set_low(self, pin):
@@ -786,9 +818,9 @@ class Eyes:
 		Sets the speciied 'pin' LOW, just before every Capture.
 		'''
 		mask = 1 << pin
-		self.fd.write(chr(SETACTION))
-		self.fd.write(chr(ACLR))
-		self.fd.write(chr(mask))
+		self.dwrite(chr(SETACTION))
+		self.dwrite(chr(ACLR))
+		self.dwrite(chr(mask))
 		self.fd.read(1)
 
 	def enable_pulse_high(self, pin):
@@ -797,9 +829,9 @@ class Eyes:
 		width is specified by the set_pulsewidth() function.
 		'''
 		mask = 1 << pin
-		self.fd.write(chr(SETACTION))
-		self.fd.write(chr(APULSEHI))
-		self.fd.write(chr(mask))
+		self.dwrite(chr(SETACTION))
+		self.dwrite(chr(APULSEHI))
+		self.dwrite(chr(mask))
 		self.fd.read(1)
 
 	def enable_pulse_low(self, pin):
@@ -807,9 +839,9 @@ class Eyes:
 		Generate a LOW TRUE Pulse on the speciied 'pin', just before every Capture.
 		'''
 		mask = 1 << pin
-		self.fd.write(chr(SETACTION))
-		self.fd.write(chr(APULSELO))
-		self.fd.write(chr(mask))
+		self.dwrite(chr(SETACTION))
+		self.dwrite(chr(APULSELO))
+		self.dwrite(chr(mask))
 		self.fd.read(1)
 
 		
@@ -820,8 +852,8 @@ class Eyes:
 		Sets the 'pulse_polarity' parameter for pulse2rtime()
 		pol = 0 means HIGH TRUE pulse 
 		'''
-		self.fd.write(chr(SETPULSEPOL))
-		self.fd.write(chr(pol))
+		self.dwrite(chr(SETPULSEPOL))
+		self.dwrite(chr(pol))
 		res = self.fd.read(1)
 		if res == 'D':
 			self.pulse_pol = pol
@@ -831,8 +863,8 @@ class Eyes:
 		Sets the 'pulse_width' parameter for pulse2rtime() command. 
 		Also used by usound_time() and the elable_pulse_high/low() functions
 		'''
-		self.fd.write(chr(SETPULSEW))
-		self.fd.write(chr(width))
+		self.dwrite(chr(SETPULSEWID))
+		self.dwrite(chr(width))
 		res = self.fd.read(1)
 		if res == 'D':
 			self.pulse_width = width
@@ -844,7 +876,7 @@ class Eyes:
 		from a Pulse on ID1 to a signal on T15, in microseconds. 
 		Use set_pulsewidth() to set the width to 13 microseconds.
 		'''
-		self.fd.write(chr(USOUND))
+		self.dwrite(chr(USOUND))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'Echo error = ',res
@@ -860,6 +892,8 @@ class Eyes:
 		First argument (pin1) is encoded in the HIGH half.
         for example pin1 = 2 , pin2 = 0, mask = 0010:0001
 		'''
+		if pin1 > 4 or pin2 > 4:
+			return -1.0
 		if pin1 == 4:        	# Analog Comparator
 			hi = 0
 		else:
@@ -870,8 +904,8 @@ class Eyes:
 		else:
 			low  = 1 << pin2
 		mask = hi | low;
-		self.fd.write(chr(cmd))
-		self.fd.write(chr(mask))
+		self.dwrite(chr(cmd))
+		self.dwrite(chr(mask))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'Time Measurement call Error. CMD = ', cmd, res
@@ -956,9 +990,9 @@ class Eyes:
 			mask = 0
 		else:
 			mask = 1 << pin
-		self.fd.write(chr(MULTIR2R))
-		self.fd.write(chr(mask))
-		self.fd.write(chr(skipcycles))
+		self.dwrite(chr(MULTIR2R))
+		self.dwrite(chr(mask))
+		self.dwrite(chr(skipcycles))
 		if self.fd.read(1) != 'D':
 			return -1.0
 		res = self.fd.read(3)
@@ -970,8 +1004,8 @@ class Eyes:
 		'''
 		Route the specified ADC channel to the Analog Comparator Input (AIN-)
 		'''
-		self.fd.write(chr(ADC2CMP))
-		self.fd.write(chr(ch))
+		self.dwrite(chr(ADC2CMP))
+		self.dwrite(chr(ch))
 		self.fd.read(1)
 
 #----------------------------- Simple Digital I/O functions ----------------------------
@@ -979,15 +1013,15 @@ class Eyes:
 		'''
 		Writes  a 2 bit number to the Digital Outputs
 		'''
-		self.fd.write(chr(DIGOUT))
-		self.fd.write(chr(val))
+		self.dwrite(chr(DIGOUT))
+		self.dwrite(chr(val))
 		self.fd.read(1)
 
 	def read_inputs(self):
 		'''
 		Gets a 4 bit number representing the Digital Input voltage Levels
 		'''
-		self.fd.write(chr(DIGIN))
+		self.dwrite(chr(DIGIN))
 		res = self.fd.read(1)
 		if res != 'D':
 			print 'DIGIN error'
@@ -997,27 +1031,27 @@ class Eyes:
 
 #-----------DIRECT PORT ACCESS FUNCTIONS (Use only if you know what you are doing)---------
 	def set_ddr(self, port, direc):
-		self.fd.write(chr(SETDDR))           
-		self.fd.write(chr(port))	 # 0 to 3 for A,B,C and D
-		self.fd.write(chr(direc))
+		self.dwrite(chr(SETDDR))           
+		self.dwrite(chr(port))	 # 0 to 3 for A,B,C and D
+		self.dwrite(chr(direc))
 		self.fd.read(1)
 		return
 
 	def set_port(self, port, val):
-		self.fd.write(chr(SETPORT))           
-		self.fd.write(chr(port))	 # 0 to 3 for A,B,C and D
-		self.fd.write(chr(val))
+		self.dwrite(chr(SETPORT))           
+		self.dwrite(chr(port))	 # 0 to 3 for A,B,C and D
+		self.dwrite(chr(val))
 		self.fd.read(1)
 		return
 
 	def get_port(self, port):
-		self.fd.write(chr(SETPORT))           
-		self.fd.write(chr(port))	 # 0 to 3 for A,B,C and D
+		self.dwrite(chr(SETPORT))           
+		self.dwrite(chr(port))	 # 0 to 3 for A,B,C and D
 		self.fd.read(1)
 		data = self.fd.read(1)     	 # get the status byte only
 		return ord(data)
 
-#--------------------------------- eyeutils.py ------------------------------
+#--------------------------------- may go to eyeutils.py ------------------------------
 	def minimum(self,va):
 		vmin = 1.0e10		# need to change
 		for v in va:
@@ -1078,3 +1112,5 @@ class Eyes:
 
 
 
+
+
