@@ -14,14 +14,6 @@ try:
 except:
 	EYEMATH = False
 
-def abs_path():
-	name = sys.argv[0]
-	dirname = os.path.dirname(name)
-	print dirname
-	if dirname != '':
-		return os.path.dirname(name) + os.sep 
-	else:
-		return '.' + os.sep
 
 WIDTH  = 555.0 * 0.8
 HEIGHT = 677.0 * 0.8
@@ -116,11 +108,16 @@ class eyePanel:
 		self.border = BORDER				# Top and bottom self.border
 		self.fw = float(width)/12.7			# field width
 		self.fh = float(height - 2 * self.border)/16	# field height
-		im = Image.open(abs_path()+ picture)
+		im = Image.open(eyeplot.abs_path()+ picture)
 		im = im.resize((int(width),int(height)))
 		self.image = ImageTk.PhotoImage(im)
 		self.panel = Canvas(parent, width = width, height = height)
 		self.panel.create_image(0,0,image = self.image, anchor = NW)
+
+		self.popup = Button(text = 'EXPERIMENTS', bg=pgreen)
+		self.popup.bind("<ButtonRelease-1>", self.pop_expt_menu)
+		self.panel.create_window(width/2, height-20, window = self.popup, anchor = CENTER)
+		
 		self.panel.bind("<ButtonRelease-1>", self.clicked)
 		self.panel.bind("<ButtonRelease-3>", self.pop_expt_menu)
 		self.panel.pack(side=TOP, anchor=SW)
@@ -297,7 +294,7 @@ class eyePanel:
 		a,i = self.ifromxy(e)
 		if a == 1:
 			showhelp(help[i])
-		print e.x, e.y, a, i
+		#print e.x, e.y, a, i
 
 
 	def duty_cycle(self):
@@ -363,7 +360,7 @@ class eyePanel:
 					rms = self.eye.rms(v)
 					f0 = fa[1][1]*1000
 					ph0 = fa[1][2]
-					s += 'CH0 : %5.1f V , %5.1f Hz '%(rms, f0)
+					s += 'CH0 : %5.2f V , %5.2f Hz '%(rms, f0)
 				else:
 					s += 'CH0: no signal '
 				fb = eyemath.fit_sine(tt,vv)
@@ -372,7 +369,7 @@ class eyePanel:
 					rms = self.eye.rms(vv)
 					f1 = fb[1][1]*1000
 					ph1 = fb[1][2]
-					s = s + '| CH1 %5.1f V , %5.1f Hz'%(rms, f1)
+					s = s + '| CH1 %5.2f V , %5.2f Hz'%(rms, f1)
 					if fa != None and abs(f0-f1) < f0*0.1:
 						s = s + ' | dphi= %5.1f'%( (ph1-ph0)*180.0/math.pi)
 				else:
@@ -487,12 +484,14 @@ def lissa_mode():
 		g.setWorld(0,-5*VPERDIV, NP * delay * 0.001, 5*VPERDIV,'mS','V')	# Restore old scale
 		lissa = False
 
-for k in range(3):
-	pe = eyes.open()			# Get the hardware connection
-	print pe
-	if pe != None:
+#-----------------------------main program starts here-----------------------------
+for k in range(20):		# Test the hardware availability by by running another program.
+	stat,out = commands.getstatusoutput('python '+ eyeplot.abs_path() + 'hwtest.py')
+	print stat
+	if stat == 0:
 		break
 
+pe = eyes.open()			# Try several times to make a connection
 root = Tk()
 
 left = Frame(root)			# Divide root window into Left and Right
@@ -574,6 +573,7 @@ expts = [
 ['Diode IV','diode_iv'],
 ['LED IV','LED_iv'],
 ['Transistor CE','transistor'],
+['Frequency Response','freq-response'],
 ['Velocity of Sound' , 'velocity-sound'],
 ['Interference of Sound' , 'interference-sound'],
 ['Photo-Transistor CE','phototransistor'],
@@ -582,6 +582,7 @@ expts = [
 ['Pendulum Wavefrorm','pendulum'],
 ['40 kHz Piezo TOF','usound_tof'],
 ['PT100 Sensor', 'pt100'],
+['Temp Comptroller', 'temp-controller'],
 ['Data Logger', 'logger'],
 ['CRO','cro'],
 ['AM and FM', 'amfm'],
@@ -594,13 +595,13 @@ def run_expt(expt):
 	global w
 	if os.name == 'nt':		# For windows OS
 		w.eye.fd.close()	# Close hardware port
-		cmd = sys.executable + ' ' + abs_path() + expt+'.py'
+		cmd = sys.executable + ' ' + eyeplot.abs_path() + expt+'.py'
 		os.system(cmd)
 		w.eye = eyes.open()	# Open hardware port again
 		showhelp('Finished ' + expt)
 	else:
 		#print abs_path() + expt+'.py'
-		stat,out = commands.getstatusoutput('python '+ abs_path() + expt+'.py')
+		stat,out = commands.getstatusoutput('python '+ eyeplot.abs_path() + expt+'.py')
 		if stat != 0:
 			showhelp(out)
 		else:
@@ -624,9 +625,7 @@ else:
 	root.title('EYES Hardware found on ' + str(pe.device))
 	pe.write_outputs(0)
 	pe.disable_actions()
-	pe.load_calib(0)
-	pe.load_calib(1)
-	pe.load_calib(8)
+	pe.loadall_calib()
 	root.after(TIMER,w.update)
 root.mainloop()
   
