@@ -4,7 +4,7 @@ Author  : Ajith Kumar B.P, bpajith@gmail.com
 License : GNU GPL version 3
 '''
 from Tkinter import *
-import expeyes.eyes as eyes
+import expeyes.eyes as eyes, expeyes.eyeplot as eyeplot
 
 
 def get_tof():
@@ -32,6 +32,7 @@ res.pack(side = TOP, anchor = N)
 b5 = Button(cf, text = 'QUIT', command = sys.exit)
 b5.pack(side = TOP, anchor = N)
 
+eyeplot.pop_image('pics/ultra-sound.png', 'Velocity of Sound, 40kHz')
 root.title('Velocity of Sound')
 root.mainloop()
 
